﻿using System.Drawing;
using System.Windows.Forms;

using DevExpress.Utils;
using DevExpress.XtraBars.Docking2010;
using DevExpress.XtraBars.Docking2010.Views;
using DevExpress.XtraBars.Docking2010.Views.WindowsUI;
using DevExpress.XtraEditors;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 윈도우즈 UI 뷰
        /// </summary>
        private WindowsUIView windowsUIView;

        /// <summary>
        /// 핍업 플라이아웃
        /// </summary>
        private Flyout popupFlyout;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            Padding = new Padding(10);

            #region 윈도우즈 UI 뷰를 설정한다.

            this.windowsUIView = new WindowsUIView();

            this.windowsUIView.AddTileWhenCreatingDocument           = DefaultBoolean.False;
            this.windowsUIView.FlyoutProperties.Appearance.BackColor = Color.DodgerBlue;

            #endregion
            #region 문서 관리자를 설정한다.

            DocumentManager documentManager = new DocumentManager();

            documentManager.ContainerControl = this;
            documentManager.View             = windowsUIView;

            #endregion
            #region 이벤트를 설정한다.

            this.windowsUIView.QueryControl                 += windowsUIView_QueryControl;
            this.windowsUIView.QueryStartupContentContainer += windowsUIView_QueryStartupContentContainer;
            this.windowsUIView.FlyoutHidden                 += windowsUIView_FlyoutHidden;

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 윈도우즈 UI 뷰 컨트롤 질의하기 - windowsUIView_QueryControl(sender, e)

        /// <summary>
        /// 윈도우즈 UI 뷰 컨트롤 질의하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void windowsUIView_QueryControl(object sender, QueryControlEventArgs e)
        {
            if(e.Document.ControlName == "PopupControl")
            {
                e.Control = new PopupControl();
            }
            else
            {
                WindowsUIView view = sender as WindowsUIView;

                LabelControl labelControl = new LabelControl();

                labelControl.Dock                              = DockStyle.Fill;
                labelControl.AutoSizeMode                      = LabelAutoSizeMode.None;
                labelControl.Appearance.TextOptions.HAlignment = HorzAlignment.Center;
                labelControl.Appearance.TextOptions.VAlignment = VertAlignment.Center;
                labelControl.Appearance.Font                   = new Font("나눔고딕코딩", 24f, FontStyle.Bold);
                labelControl.Text                              = $"{e.Document.Header} Content";

                e.Control = labelControl;
            }
        }

        #endregion
        #region 윈도우즈 UI 뷰 시작 컨텐트 컨테이너 질의하기 - windowsUIView_QueryStartupContentContainer(sender, e)

        /// <summary>
        /// 윈도우즈 UI 뷰 시작 컨텐트 컨테이너 질의하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void windowsUIView_QueryStartupContentContainer(object sender, QueryContentContainerEventArgs e)
        {
            #region 타일 컨테이너를 설정한다.

            TileContainer tileContainer = new TileContainer();

            tileContainer.Caption                  = "Tile Container";
            tileContainer.Properties.ItemSize      = 150;
            tileContainer.Properties.ShowGroupText = DefaultBoolean.True;

            Document document1 = new Document(); document1.Header = "Document 1";
            Document document2 = new Document(); document2.Header = "Document 2";
            Document document3 = new Document(); document3.Header = "Document 3";
            Document document4 = new Document(); document4.Header = "Document 4";

            Tile tile1 = new Tile();
            Tile tile2 = new Tile();
            Tile tile3 = new Tile();
            Tile tile4 = new Tile();

            tile1.Group               = "Group 1";
            tile1.Properties.ItemSize = TileItemSize.Medium;
            tile1.Document            = document1;

            tile1.Click += tile1_Click;

            tile2.Group               = "Group 1";
            tile2.Properties.ItemSize = TileItemSize.Medium;
            tile2.Document            = document2;

            tile3.Group               = "Group 1";
            tile3.Properties.ItemSize = TileItemSize.Medium;
            tile3.Document            = document3;

            tile4.Group               = "Group 2";
            tile4.Properties.ItemSize = TileItemSize.Medium;
            tile4.Document            = document4;

            tile1.Elements.Add(new TileItemElement { Text = "Document 1" });
            tile2.Elements.Add(new TileItemElement { Text = "Document 2" });
            tile3.Elements.Add(new TileItemElement { Text = "Document 3" });
            tile4.Elements.Add(new TileItemElement { Text = "Document 4" });

            tileContainer.Items.AddRange(new BaseTile[] { tile1, tile2, tile3, tile4 });

            (sender as WindowsUIView).ContentContainers.Add(tileContainer);

            e.ContentContainer = tileContainer;

            #endregion
            #region 팝업 문서/플라이아웃을 설정한다.

            Document popupDocument = new Document();

            popupDocument.ControlName     = "PopupControl";
            popupDocument.ControlTypeName = "TestProject.PopupControl";

            this.windowsUIView.Documents.Add(popupDocument);

            FlyoutAction popupAction = new FlyoutAction()
            {
                Caption     = "고객 추가",
                Description = "입력한 자료를 추가하시겠습니까?"
            };

            popupAction.Commands.Clear();

            popupAction.Commands.Add(FlyoutCommand.OK    );
            popupAction.Commands.Add(FlyoutCommand.Cancel);

            this.popupFlyout = new Flyout();

            this.popupFlyout.Name             = "popupFlyout";
            this.popupFlyout.Properties.Style = FlyoutStyle.MessageBox;
            this.popupFlyout.Document         = popupDocument;
            this.popupFlyout.Action           = popupAction;

            this.windowsUIView.ContentContainers.Add(this.popupFlyout);

            #endregion
        }

        #endregion
        #region 윈도우즈 UI 뷰 플라이아웃 숨긴 경우 처리하기 - windowsUIView_FlyoutHidden(sender, e)

        /// <summary>
        /// 윈도우즈 UI 뷰 플라이아웃 숨긴 경우 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void windowsUIView_FlyoutHidden(object sender, FlyoutResultEventArgs e)
        {
            WindowsUIView view = sender as WindowsUIView;

            Document document = view.ActiveDocument as Document;

            if(document.ControlName == "PopupControl")
            {
                PopupControl popupControl = document.Control as PopupControl;

                if(e.Result == DialogResult.OK)
                {
                    if(!string.IsNullOrEmpty(popupControl.CustomerName))
                    {
                        XtraMessageBox.Show
                        (
                            this,
                            popupControl.CustomerName,
                            "INFORMATION",
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Information
                        );
                    }
                }

                popupControl.CustomerName = string.Empty;
                popupControl.Birthday     = string.Empty;
                popupControl.MailAddress  = string.Empty;
            }
        }

        #endregion

        #region 타일 1 클릭시 처리하기 - tile1_Click(sender, e)

        /// <summary>
        /// 타일 1 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void tile1_Click(object sender, TileClickEventArgs e)
        {
            this.windowsUIView.ActivateContainer(this.popupFlyout);

            e.Handled = true;
        }

        #endregion
    }
}